/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     | Website:  https://openfoam.org
    \\  /    A nd           | Copyright (C) 2016-2019 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::RBD::rigidBodySolvers::RK5

Description
    Fifth-order Cash-Karp Runge-Kutta scheme

SourceFiles
    RK5.C

\*---------------------------------------------------------------------------*/

#ifndef RK5_H
#define RK5_H

#include "rigidBodySolver.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace RBD
{
namespace rigidBodySolvers
{

/*---------------------------------------------------------------------------*\
                           Class RK5 Declaration
\*---------------------------------------------------------------------------*/

class RK5
:
    public rigidBodySolver
{
    // Private Data

    const scalar c2_, c3_, c4_, c5_, c6_;
    const scalar a21_;
    const scalar a31_, a32_;
    const scalar a41_, a42_, a43_;
    const scalar a51_, a52_, a53_, a54_;
    const scalar a61_, a62_, a63_, a64_, a65_;
    const scalar b1_, b2_, b3_, b4_, b5_, b6_;

    scalarField k1_;
    scalarField k2_;
    scalarField k3_;
    scalarField k4_;
    scalarField k5_;

public:

    //- Runtime type information
    TypeName("RK5");


    // Constructors

        //- Construct for the given body from dictionary
        RK5
        (
            rigidBodyMotion& body,
            const dictionary& dict
        );


    //- Destructor
    virtual ~RK5();


    // Member Functions

        //- Integrate the rigid-body motion for one time-step
        virtual void solve
        (
            const scalarField& tau,
            const Field<spatialVector>& fx
        );
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace rigidBodySolvers
} // End namespace RBD
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
